/*!
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Condition_1 = require("./Condition");
const Type_1 = require("./Type");
const Converters_1 = require("./Converters");
const Operator_1 = require("./Operator");
const OperatorOverload_1 = require("./OperatorOverload");
class QueryBuilder {
    constructor() {
        this.types = new Map();
        this.typeAliases = new Map();
        this.operators = new Map();
        this.conditions = new Map();
        this.filters = new Map();
        // Default conditions
        this.registerCondition(new Condition_1.Condition("AND", x => x.every(y => y)));
        this.registerCondition(new Condition_1.Condition("OR", x => x.some(y => y)));
        // Default types
        this.registerType(new Type_1.Type("string", Converters_1.convertString));
        this.registerType(new Type_1.Type("number", Converters_1.convertNumber));
        this.registerType(new Type_1.Type("datetime", Converters_1.convertDate));
        this.registerType(new Type_1.Type("boolean", Converters_1.convertBoolean));
        // Special types (used for some operators)
        this.registerType(new Type_1.Type("stringarray", Converters_1.convertStringArray));
        this.registerType(new Type_1.Type("numberarray", Converters_1.convertNumberArray));
        this.registerType(new Type_1.Type("datetimearray", Converters_1.convertDateArray));
        // Filters use a diferent typeset, so we have to alias them to their real type
        this.registerTypeAlias("string", "string");
        this.registerTypeAlias("integer", "number");
        this.registerTypeAlias("double", "number");
        this.registerTypeAlias("date", "datetime");
        this.registerTypeAlias("time", "datetime");
        this.registerTypeAlias("datetime", "datetime");
        this.registerTypeAlias("boolean", "boolean");
        // Default operators
        this.registerOperator(new Operator_1.Operator("equal", [
            new OperatorOverload_1.OperatorOverload("string", "string", (a, b) => a === b),
            new OperatorOverload_1.OperatorOverload("number", "number", (a, b) => a === b),
            new OperatorOverload_1.OperatorOverload("boolean", "boolean", (a, b) => a === b),
            new OperatorOverload_1.OperatorOverload("datetime", "datetime", (a, b) => a.getTime() === b.getTime())
        ]));
        this.registerOperator(new Operator_1.Operator("not_equal", [
            new OperatorOverload_1.OperatorOverload("string", "string", (a, b) => a !== b),
            new OperatorOverload_1.OperatorOverload("number", "number", (a, b) => a !== b),
            new OperatorOverload_1.OperatorOverload("boolean", "boolean", (a, b) => a !== b),
            new OperatorOverload_1.OperatorOverload("datetime", "datetime", (a, b) => a.getTime() !== b.getTime())
        ]));
        this.registerOperator(new Operator_1.Operator("in", [
            new OperatorOverload_1.OperatorOverload("number", "numberarray", (a, b) => b.includes(a)),
            new OperatorOverload_1.OperatorOverload("string", "stringarray", (a, b) => b.includes(a))
        ]));
        this.registerOperator(new Operator_1.Operator("not_in", [
            new OperatorOverload_1.OperatorOverload("number", "numberarray", (a, b) => !b.includes(a)),
            new OperatorOverload_1.OperatorOverload("string", "stringarray", (a, b) => !b.includes(a))
        ]));
        this.registerOperator(new Operator_1.Operator("less", [
            new OperatorOverload_1.OperatorOverload("number", "number", (a, b) => a < b),
            new OperatorOverload_1.OperatorOverload("datetime", "datetime", (a, b) => a.getTime() < b.getTime())
        ]));
        this.registerOperator(new Operator_1.Operator("less_or_equal", [
            new OperatorOverload_1.OperatorOverload("number", "number", (a, b) => a <= b),
            new OperatorOverload_1.OperatorOverload("datetime", "datetime", (a, b) => a.getTime() <= b.getTime())
        ]));
        this.registerOperator(new Operator_1.Operator("greater", [
            new OperatorOverload_1.OperatorOverload("number", "number", (a, b) => a > b),
            new OperatorOverload_1.OperatorOverload("datetime", "datetime", (a, b) => a.getTime() > b.getTime())
        ]));
        this.registerOperator(new Operator_1.Operator("greater_or_equal", [
            new OperatorOverload_1.OperatorOverload("number", "number", (a, b) => a >= b),
            new OperatorOverload_1.OperatorOverload("datetime", "datetime", (a, b) => a.getTime() >= b.getTime())
        ]));
        this.registerOperator(new Operator_1.Operator("between", [
            new OperatorOverload_1.OperatorOverload("number", "numberarray", (a, b) => (!(0 in b) || a > b[0]) && (!(1 in b) || a < b[1])),
            new OperatorOverload_1.OperatorOverload("datetime", "datetimearray", (a, b) => (!(0 in b) || a > b[0]) && (!(1 in b) || a < b[1]))
        ]));
        this.registerOperator(new Operator_1.Operator("not_between", [
            new OperatorOverload_1.OperatorOverload("number", "numberarray", (a, b) => !(!(0 in b) || a > b[0]) && (!(1 in b) || a < b[1])),
            new OperatorOverload_1.OperatorOverload("datetime", "datetimearray", (a, b) => !(!(0 in b) || a > b[0]) && (!(1 in b) || a < b[1]))
        ]));
        this.registerOperator(new Operator_1.Operator("begins_with", [
            new OperatorOverload_1.OperatorOverload("string", "string", (a, b) => a.startsWith(b))
        ]));
        this.registerOperator(new Operator_1.Operator("not_begins_with", [
            new OperatorOverload_1.OperatorOverload("string", "string", (a, b) => !a.startsWith(b))
        ]));
        this.registerOperator(new Operator_1.Operator("contains", [
            new OperatorOverload_1.OperatorOverload("string", "string", (a, b) => a.includes(b))
        ]));
        this.registerOperator(new Operator_1.Operator("not_contains", [
            new OperatorOverload_1.OperatorOverload("string", "string", (a, b) => !a.includes(b))
        ]));
        this.registerOperator(new Operator_1.Operator("ends_with", [
            new OperatorOverload_1.OperatorOverload("string", "string", (a, b) => a.endsWith(b))
        ]));
        this.registerOperator(new Operator_1.Operator("not_ends_with", [
            new OperatorOverload_1.OperatorOverload("string", "string", (a, b) => !a.endsWith(b))
        ]));
        this.registerOperator(new Operator_1.Operator("is_empty", [
            new OperatorOverload_1.OperatorOverload("string", "*", (a, b) => a.length === 0)
        ]));
        this.registerOperator(new Operator_1.Operator("is_not_empty", [
            new OperatorOverload_1.OperatorOverload("string", "*", (a, b) => a.length !== 0)
        ]));
        this.registerOperator(new Operator_1.Operator("is_null", [
            new OperatorOverload_1.OperatorOverload("*", "*", (a, b) => a === null)
        ]));
        this.registerOperator(new Operator_1.Operator("is_not_null", [
            new OperatorOverload_1.OperatorOverload("*", "*", (a, b) => a !== null)
        ]));
    }
    registerType(type) {
        this.types.set(type.id, type);
    }
    registerTypeAlias(fieldTypeId, dataTypeId) {
        this.typeAliases.set(fieldTypeId, dataTypeId);
    }
    registerOperator(operator) {
        this.operators.set(operator.id, operator);
    }
    registerCondition(condition) {
        this.conditions.set(condition.id, condition);
    }
    registerFilter(filter) {
        this.filters.set(filter.id, filter);
    }
    runQuery(entity, rules, validationOptions) {
        return this.evaluateRuleOrGroup(entity, rules, validationOptions);
    }
    evaluateRuleOrGroup(entity, ruleOrGroup, vopts) {
        if (isValidGroup(ruleOrGroup))
            return this.evaluateGroup(entity, ruleOrGroup, vopts);
        if (isValidRule(ruleOrGroup))
            return this.evaluateRule(entity, ruleOrGroup, vopts);
        else
            throw new Error("Invalid object in ruleset.");
    }
    evaluateRule(entity, rule, vopts) {
        if (vopts) {
            vopts.ruleCount = (vopts.ruleCount || 0) + 1;
            if (vopts.maxRules && vopts.ruleCount > vopts.maxRules) {
                throw new Error(`Validation Error: Too many rules. The maximum number of rules is ${vopts.maxRules}.`);
            }
        }
        const filter = this.filters.get(rule.id);
        if (!filter)
            throw new Error(`Invalid filter '${rule.id}'.`);
        const filterDataType = this.typeAliases.get(filter.typeAliasId);
        if (!filterDataType) {
            throw new Error(`Could not get data type for filter '${rule.id}': filter type '${filter.typeAliasId}' is not mapped to an internal data type.`);
        }
        let operator = this.operators.get(rule.operator);
        if (!operator) {
            operator = this.operators.get('*');
            if (!operator) {
                throw new Error(`Invalid operator '${rule.operator}'.`);
            }
        }
        const overload = operator.getOverloadFor(filterDataType);
        if (!overload) {
            throw new Error(`Operator '${rule.operator}' can not be used with type '${filterDataType}'.`);
        }
        let valueDataType = null;
        if (overload.rightHandTypeId !== '*') {
            valueDataType = this.types.get(overload.rightHandTypeId);
            if (!valueDataType) {
                throw new Error(`Invalid operator '${rule.operator}': The right-hand side's data type '${overload.rightHandTypeId}' does not exist.`);
            }
        }
        if (vopts && vopts.validateOnly)
            return true;
        const leftValue = filter.getValue(entity);
        const rightValue = valueDataType ? valueDataType.convert(rule.value) : rule.value;
        return overload.evaluate(leftValue, rightValue);
    }
    evaluateGroup(entity, group, vopts) {
        if (vopts) {
            vopts.groupCount = (vopts.groupCount || 0) + 1;
            if (vopts.maxGroups && vopts.groupCount > vopts.maxGroups) {
                throw new Error(`Validation Error: Too many groups. The maximum number of groups is ${vopts.maxRulesPerGroup}.`);
            }
            vopts.currentDepth = (vopts.currentDepth || 0) + 1;
            if (vopts.maxDepth && vopts.currentDepth > vopts.maxDepth) {
                throw new Error(`Validation Error: Too many nested groups. The maximum depth is ${vopts.maxRulesPerGroup}.`);
            }
        }
        const condition = this.conditions.get(group.condition);
        if (!condition) {
            throw new Error(`Invalid condition '${group.condition}'.`);
        }
        if (vopts && vopts.maxRulesPerGroup && group.rules.length > vopts.maxRulesPerGroup) {
            throw new Error(`Validation Error: Too many rules in group. The maximum rules per group is ${vopts.maxRulesPerGroup}, you have ${group.rules.length}.`);
        }
        const rtn = condition.evaluate(group.rules.map(r => this.evaluateRuleOrGroup(entity, r, vopts)));
        if (vopts)
            vopts.currentDepth = (vopts.currentDepth || 0) - 1;
        return rtn;
    }
}
exports.QueryBuilder = QueryBuilder;
function isValidGroup(thing) {
    return 'condition' in thing
        && 'rules' in thing
        && Array.isArray(thing.rules);
}
function isValidRule(thing) {
    return 'id' in thing
        && 'operator' in thing
        && 'value' in thing;
}
